// base.cpp
//
// mwanova - Multi-Way Analysis of Variance.
// Copyright (C) 2001  Antonio Santos (amsantos@fc.up.pt)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// *************************************************************************
// 


#include <iostream.h>
#include <string.h>
#include <iomanip.h>
#include "base.h"

//************************************************************************//
//**************************** PUBLIC FUNCTIONS **************************//
//************************************************************************//

base::base()
{
 strcpy(datafilename,"");	// Data file name
 noanova=false;			// Do not compute anova
 verbose=false;			// Be verbose
 orthogonal=false;		// Do not use model
 ctrules=false;			// Display Cornfield Tukey Rules
 stdinput=false;		// Use standard input instead of file
 pretransf=NOTRANSF;		// Pre-transformation
 transf=NOTRANSF;		// Apply transformation to data
 do_debug=false;		// Debug - very verbose mode
 mtable=false;			// Show means table
 homogeneity=false;             // Show tests of homogeneity
}

base::~base()
{
}

bool base::be_verbose()
{
 return verbose;
}

bool base::debug()
{
 return do_debug;
}

  
bool base::do_anova()
{
 return !noanova;
}
  
bool base::show_orthogonal()
{
 return orthogonal;
}

int  base::pretransform()
{
 return pretransf;
}

int  base::transformation()
{
 return transf;
}

bool base::show_ctrules()
{
 return ctrules;
}
  
bool base::show_mtable()
{
 return mtable;
} 

bool base::show_var_tests()
{
 return homogeneity;
}

bool base::is_stdinput()
{
 return stdinput;
}
 
const char *base::data_file_name()
{
 return datafilename;
}

void base::parse_args(int argc, char *argv[])
{
 int 	i;
 bool 	InputFileExists=false;
 char	token[20];
 
 i=1;
 do{
  if(argv[i][0]=='-'){
   switch(argv[i][1]){
    case 'a': mtable=true; i++; break;         // output means table
    case 'n': noanova=true; i++; break;        // do not ouput anova table
    case 'v': verbose=true; i++; break;        // be verbose
    case 'r': ctrules=true; i++; break;        // output Cornfield-Tukey rules table
    case 'd': do_debug=true; i++; break;       // debuggin info
    case 'h': homogeneity=true; i++; break;    // show homogeneity tests
    case 'o': orthogonal=true; i++; break;    // show homogeneity tests
    case 'f': i++;                             // data file name
              if((i<argc)&&(argv[i][0]!='-')){
               strcpy(datafilename,argv[i]);
	       InputFileExists=true;            
	       i++;
	      }
	      else noanova=true;  
	      break;
    case 'p': i++;                             // type of pre-transformation
              if((i<argc)&&(argv[i][0]!='-')){
               strcpy(token,argv[i]);
	       if(strcmp(token,"sqrt")==0) pretransf=SQRTRANSF;
	       if(strcmp(token,"log")==0) pretransf=LOGTRANSF;
	       if(strcmp(token,"ln")==0) pretransf=LNTRANSF;
	       if(strcmp(token,"arcsin")==0) pretransf=ARCSTRANSF;
	       if(strcmp(token,"asin")==0) pretransf=ARCSTRANSFRAD;
	       if(strcmp(token,"mult")==0) pretransf=MULT100;
	       if(strcmp(token,"div")==0) pretransf=DIV100;
	       i++;
	      } 
	      else{
	       pretransf=NOTRANSF;
	       i++;
	      }
    case 't': i++;                             // type of transformation
              if((i<argc)&&(argv[i][0]!='-')){
               strcpy(token,argv[i]);
	       if(strcmp(token,"sqrt")==0) transf=SQRTRANSF;
	       if(strcmp(token,"log")==0) transf=LOGTRANSF;
	       if(strcmp(token,"ln")==0) transf=LNTRANSF;
	       if(strcmp(token,"arcsin")==0) transf=ARCSTRANSF;
	       if(strcmp(token,"asin")==0) transf=ARCSTRANSFRAD;
	       if(strcmp(token,"mult")==0) transf=MULT100;
	       if(strcmp(token,"div")==0) transf=DIV100;
	       i++;
	      } 
	      else{
	       transf=NOTRANSF;
	       i++;
	      } 
	      break;	      
    default: i++; break;	      
   } 
  }
  else i++;  
 }while(i<argc);
 
 
 if((!InputFileExists)&&(!noanova)) stdinput=true;   
}


void base::help()
{
 cout << "mwanova - Multi-Way Analysis of Variance  Version 2.0-pre1" << endl;
 cout << "Copyright (C)  2001  Antonio Santos (amsantos@fc.up.pt)" << endl << endl;
 cout << "This program is distributed under the GNU General Public License" << endl;
 cout << "See the COPYING file for details" << endl << endl;
 cout << "Program usage:" << endl;
 cout << "  mwanova -f DataFile [options]" << endl;
 cout << "  cat DataFile | mwanova -f [options]" << endl << endl;
 cout << "OPTIONS:" << endl;
 cout << "  -a  output means table      -n  do not output anova table" << endl;
 cout << "  -o  output orthogonal model -h  homogeneity tests" << endl;
 cout << "  -v  be verbose              -r  output Cornfield-Tukey rules table" << endl;
 cout << endl;
 cout << "  -p sqrt|log|ln|arcsin|asin|mult|div  pre-transform data " << endl;
 cout << "  -t sqrt|log|ln|arcsin|asin|mult|div  transform data " << endl;
 cout << endl;
 cout << "Read the man page for more information" << endl;
}

//------------------------------------------------------------------------//
// Draws a header with an string 'h' in the middle before outputing any   //
// data. The string is enclosed in '---'.                                 //
//------------------------------------------------------------------------//

void base::header(const char *h)
{
 unsigned int rest,i;
 char s[ROWSIZE+1];
 
 memset(s,0,ROWSIZE+1);
 memset(s,'-',ROWSIZE);
 
 if(strlen(h)<ROWSIZE){
  rest=ROWSIZE-strlen(h);
  rest/=2;
  for(i=0;i<strlen(h);i++) s[rest+i]=h[i];
 } 
 cout << s << endl;
}

//------------------------------------------------------------------------//
// Draws a footer after outputing data. The footer is a string filled     //
// with '-----------'.                                                    //
//------------------------------------------------------------------------//

void base::footer()
{
 char s[ROWSIZE+1];
 
 memset(s,0,ROWSIZE+1);
 memset(s,'-',ROWSIZE);
 cout << s << endl;
}


